;                            Software License Agreement
;
; The software supplied herewith by Microchip Technology Incorporated (the "Company")
; for its PICmicro Microcontroller is intended and supplied to you, the Companys
; customer, for use solely and exclusively on Microchip PICmicro Microcontroller
; products.
;
; The software is owned by the Company and/or its supplier, and is protected under
; applicable copyright laws. All rights are reserved. Any use in violation of the
; foregoing restrictions may subject the user to criminal sanctions under applicable
; laws, as well as to civil liability for the breach of the terms and conditions of
; this license.
;
; THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, WHETHER EXPRESS,
; IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
; MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE
; COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
; CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
;
; ###############################################################################
; *******************************************************************************
;   PIC16CE62x interface functions test program.                                *
;   Written and Tested using MPASM V1.40                                        *
;********************************************************************************
;   Filename:		CHECK62X.ASM                                            *
;   Date:               23 February 2001                                        *
;   File Version:       1.10                                                    *
;********************************************************************************
;   Functions:                                                                  *
;                                                                               *
;********************************************************************************
;   Revision History:                                                           *
;      V1.00 - Initial release                                                  *
;      V1.10 - Revised for V1.10 release of interface functions                 *
;********************************************************************************
;   Notes:                                                                      *
;   Exercises the 16CE62x interface functions.                                  *
;   Writes and reads back the varying patterns to the EEPROM.                   *
;   The writes and reads occur both in Page and Byte mode.  The                 *
;   read step is bit-banged out PORTA, bit 3 at 19.2kbaud (8N1).                *
;   This may be read by any terminal program such as Hyperterm.                 *
;********************************************************************************
; *******************************************************************************
;   PIC16CE62x interface functions test program.                                *
;   Written and Tested using MPASM V1.40                                        *
;********************************************************************************
;   Filename:		CHECK62X.ASM                                            *
;   Date:               23 February 2001                                        *
;   File Version:       1.10                                                    *
;********************************************************************************
;   Functions:                                                                  *
;                                                                               *
;********************************************************************************
;   Revision History:                                                           *
;      V1.00 - Initial release                                                  *
;      V1.10 - Revised for V1.10 release of interface functions                 *
;********************************************************************************
;   Notes:                                                                      *
;   Exercises the 16CE62x interface functions.                                  *
;   Writes and reads back the varying patterns to the EEPROM.                   *
;   The writes and reads occur both in Page and Byte mode.  The                 *
;   read step is bit-banged out PORTA, bit 3 at 19.2kbaud (8N1).                *
;   This may be read by any terminal program such as Hyperterm.                 *
;********************************************************************************
	list    p=pic16CE625
#include <P16CE625.inc>

        __config (_CP_OFF &  _WDT_OFF & _HS_OSC & _BODEN_OFF)
#include "flash62x.inc"
#include "swuart14.inc"
	extern  WRITE_PAGE
	extern	READ_PAGE
	global	printit

#define  ERRORS  PORTB,1   ; On when until an error is detected

	udata
mode    res     1
address res     1
buffer  res	10
counter	res	1

	udata_shr
printee res     1
temp    res     1

vectors code 0    
        goto    start
	nop
	nop
	nop
	nop
	code

patterntable
	movwf	temp
	movlw	high	ptstart
	movwf	PCLATH
	movlw	low	ptstart
	addwf	temp,w
	btfsc	STATUS,C
	incf	PCLATH,f
	movwf	PCL
ptstart
        retlw   0x01
        retlw   0x23
        retlw   0x45
        retlw   0x67    ;aa
        retlw   0x89    ;0f
        retlw   0xab    ;f0
        retlw   0xcd
        retlw   0xef
        retlw   0xa5

nextmode
	banksel	mode
        incf    mode,f
        movf    mode,w
        xorlw   0x08
        btfsc   STATUS,Z
        clrf    mode
	return

;****************************************************************
;       Print it
;       Prints the byte provided in W as a two digit hex value
;	Local variables (printee and temp) are stored in Shared Ram
;****************************************************************
printit
	movwf	printee
        swapf   printee,w
        andlw   0x0f
        movwf   temp
        movlw   0x0A
        subwf   temp,w
        btfsc   STATUS,C
        goto    addA
        movlw   '0'
        goto    print16s
addA    movlw   'A'-0x0A
print16s
        addwf   temp,w
	call    putcUART;

        movf    printee,w
        andlw   0x0f
        movwf   temp
        movlw   0x0A
        subwf   temp,w
        btfsc   STATUS,C
        goto    addA1
        movlw   '0'
        goto    print1s
addA1   movlw   'A'-0x0A
print1s
        addwf   temp,w
        call    putcUART
	return

;****************************************************************
;       Power up routine
;       This is the program entry point.
;****************************************************************

start   clrf    INTCON

        bsf     STATUS,RP0      ; set register page 1

	movlw	0x00
        movwf   TRISB          ; all outputs

        bcf     STATUS,RP0      ; back to register page 0
	clrf	PORTB
	movlw   0x07
        movwf   CMCON           ; set all IO digital

        bsf     ERRORS          ; turn on error bt.
        call    OpenUART
	movlw	'*'
	call	putcUART
	banksel	mode
        clrf    mode            ; set pattern mode to 0
	banksel	PORTB
nextlap
        movf    PORTB,w
        xorlw   0x04		; Flip i'm alive bit
        movwf   PORTB
	movlw	'\r'
	call	putcUART
	movlw	'\n'
	call	putcUART
	movlw	'B'
	call	putcUART
	movlw	'y'
	call	putcUART
	movlw	't'
	call	putcUART
	movlw	'e'
	call	putcUART
	movlw	'\r'
	call	putcUART
	movlw	'\n'
	call	putcUART

	call	nextmode
	call	writebybyte
	call	readbybyte
	call	readbypage

	movlw	'\r'
	call	putcUART
	movlw	'\n'
	call	putcUART
	movlw	'P'
	call	putcUART
	movlw	'a'
	call	putcUART
	movlw	'g'
	call	putcUART
	movlw	'e'
	call	putcUART
	movlw	'\r'
	call	putcUART
	movlw	'\n'
	call	putcUART

	call	nextmode
	call	writebypage
	call	readbypage
	call	readbybyte
	goto	nextlap

; ***************************************************************
writebybyte
	banksel	address
	clrf    address         ; clear the i2c address
        
nextwriteb
        movf    address,w
        xorlw   0x80
        btfsc   STATUS,Z        ; is address = 16?
   	return	        ; yes... bail out of write loop
rewriteb
        movf    address,w
        movwf   EEADDR
        movf    mode,w
        call    patterntable
	movwf	EEDATA
        call    WRITE_BYTE
        btfss   PC_OFFSET,7	; was write successful?
        goto    rewriteb	; no, try again
        incf   address,f	; bump the address
        goto    nextwriteb
	return

; ***************************************************************
writebypage
;	movlw	'W'
;	call	putcUART
	banksel	address
        clrf    address         ; clear the i2c address

nextwritep
        movf    address,w
        xorlw   0x80
        btfsc   STATUS,Z        ; is address = 80?
   	return	        ; yes... bail out of write loop

	movf    mode,w
	call    patterntable
	movwf	buffer
	movwf	buffer+1
	movwf	buffer+2
	movwf	buffer+3
	movwf	buffer+4
	movwf	buffer+5
	movwf	buffer+6
	movwf	buffer+7
rewrite
        movf    address,w
        movwf   EEADDR
	movlw	buffer
	movwf	FSR
	movlw	8
        call    WRITE_PAGE
        xorlw   0x01
        btfss   STATUS,Z             ; was write successful?
        goto    rewrite         ; no, try again
	movlw	8
        addwf   address,f       ; bump the address
        goto    nextwritep

; ***************************************************************
readbybyte
	banksel	address
        clrf    address
nextreadb
	movf    address,w
        xorlw   0x80
        btfsc   STATUS,Z        ; is address = 16?
        return			; yes... bail out of write loop
        movf    address,w
        andlw   0x0F
        btfss   STATUS,Z        ; is address = 16?
        goto    rereadb		; yes... bail out of write loop

        movlw   '\r'		; end of line
        call    putcUART
        movlw   '\n'
        call    putcUART
 	movf	address,w
 	call	printit
 	movlw	' '
 	call	putcUART

rereadb
        movf    address,w
        movwf   EEADDR
	call    READ_RANDOM
        btfss   PC_OFFSET,7	; was read successful?
	goto	rereadb

	movf	EEDATA,w
	call	printit
	movlw   ' '
	call    putcUART

        incf    address,f       ; bump the address
        goto    nextreadb
       

; ***************************************************************
readbypage
	banksel	address
        clrf    address
nextreadp
	movf    address,w
        xorlw   0x80
        btfsc   STATUS,Z        ; is address = 16?
        return			; yes... bail out of write loop

        movlw   '\r'
        call    putcUART
        movlw   '\n'
        call    putcUART
 	movf	address,w
 	call	printit
 	movlw	' '
 	call	putcUART
rereadp
        movf    address,w
        movwf   EEADDR
	movlw	buffer		; load FSR with buffer pointer
	movwf	FSR
	movlw	0x10		; read 16 bytes
	call	READ_PAGE
        xorlw   0x01
        btfss   STATUS,Z             ; was read successful?
        goto    rereadp         ; no, try again

	movlw	0x10
	movwf	counter
	movlw	buffer
	movwf	FSR
printloop
	movf	INDF,w
	call	printit
	movlw   ' '
        call    putcUART

	incf	FSR,f
	decfsz	counter,f
	goto	printloop
      
	movlw	0x10        
	addwf	address,f
        goto    nextreadp


        end

